// SERIAL2TTLdriver_reader
// Serial to TTL driver for the Rflex system
// see http://sourceforge.net/p/rflex/wiki/Home/ for more info
//
// Uses two output pins to move a stepper motor, one pin for direction and one for step.
// An input is used to read a proximity switch (position feedback from the stepping motor)
//  
// The program waits for a serial comand, if it can be recognized 
// it performs the number of steps in the given directon. 
// Acknowledgment is sent as the number of steps taken and the status of the proximity switch
// 
// Command example "STEP-012Please" will return "moved -12 steps, proximity switch is 1"
// #TODO in specified intervals the arduino tells that it is alive 
// #TODO improve acknowledgement, include line number or similar, and checksum to identify dropped commands.
// 
// version information:
// update 130702, Use analog input A0 instead of digital pin 12 to sense the proximity switch, to prevent reading false positives on small voltage spikes
// update 121214, moving the proximity switch to pin 12, hardware change ncludes voltage limitation circuit on the shield
// update 121107, recover from bad serial comand if cmd is longer than 16 char the instring buffer is set to null
// update 121024, this is the working version 12 single step/ sec with the current LabView vi driving it.
//  Checking and changing filename moving the file into the labview library for the rflex_moxa
// Update 120525 adding  save as "Serial2TTLdriver_reader_1" // to debug rows, 
// 120521 Current version _0 is debug and test file: "Serial2TTLdriver_reader_0"
// 

// VARIABLES***********************

// Serial communication
String inString = "";    // string to hold input
char inChar;              // to store the read value
int serial_speed=9600;    // 9600 default but also 19200 can work

// string handling and convert to number
String Start_cmd="STEP";    // the start of the serial comand
String End_cmd="Please";    // this is instead of a checksum
int sign_index;          // length of Start_cmd -->in setup
int  first_number_index; // in setup
int length_number=4;    // number of digits in the number including sign
int last_index;         // way to get this floating ? in setup 
String string_number;
int step_value;

// for the stepperdriver
int CCW_pin=10;
int Step_pin=11;
int Step_period=20; //ms
int half_period=Step_period/2;

// for the proximity switch
int proximity_pin = A0; // use voltage protection circuit to avoid damage, see building instructions
int Prox_switch; // mapped to 0-1024

//Acknowledge command
// Send answer in the form "moved[+ or -][steps,3digits], proximity switch is [0/1]"
// replace [] with signs and numbers, or a comma separated string?
String strAcknow1="Moved ";
int intAcknow_step;
String strAcknow2=" steps, proximity switch is ";
int intAcknow_proxi;
String strAcknowledge;

//#TODO Heartbeat
// get an acknowledge if the arduino is alive
int longTimer_last=0;
int longTimer_current;
int Interval_alive=5000;
String alive_msg1="Arduino ok and temp is ";
String alive_msg2=" degrees Celsius";

//#TODO Thermometer
// Use a simple tmp36 to read temperature
// the output is 10mV/deg with an offset of 500mV.
int temperature_pin=2; // the analoge input for the sensor
float temperature; // temperature variable
float temp_offset=0.5;
// ******************************************
// PROGRAM **********************************

// SETUP
void setup() {
  // Initialize serial communications:
  Serial.begin(serial_speed);
  // pinmode
  pinMode(CCW_pin,OUTPUT);
  pinMode(Step_pin,OUTPUT);
  pinMode(proximity_pin,INPUT);
  
  // get the start and end indexes of the cmd string
  sign_index=Start_cmd.length();
  first_number_index=sign_index+1;
  last_index=first_number_index+length_number-1;
  
}
  
// ***************************************************
// MAIN PROGRAM **************************************

void loop() {

  
// Read serial input:
  while (Serial.available() > 0) {
    int inChar = Serial.read();
         inString += (char)inChar;
        // Serial.println(inString);  // debug
    }
    
// Dephrase the input
// the imput needs a start header and a checkending
// the format of the variable is [sign]+3 digits, total 4 characters
    // wrong inString added 121107
    if (inString.length()>16){    // eg string too long
    inString="";
    }
    if (inString.startsWith(Start_cmd) && inString.endsWith(End_cmd))  { 
      // ex. "STEP-003Please" --> should give 3 steps in CCW      
      string_number=inString.substring(sign_index,last_index);
      char carray[5];
      string_number.toCharArray(carray, sizeof(carray));
      step_value = atof(carray);   // step_value range from -999 to 999  
      
 // Where the action starts
 // CCW On
       if (step_value<0)   {
       digitalWrite(CCW_pin,HIGH);
       }
        
 // STEPPING      
       for (int x=0; x<abs(step_value) ; x++)  {
         digitalWrite(Step_pin,HIGH);
          delay(half_period);    
         digitalWrite(Step_pin,LOW);
          delay(half_period);          
       }

// CCW Off
       digitalWrite(CCW_pin,LOW);
       
//Check the proximity switch
       delay(50); //ms delay to let the signal settle (spikes caused by high-amp motor?) low/high limit is 2V
       Prox_switch=analogRead(proximity_pin);


// Acknowledge
// Generate an acknowledge and output the Proximity switch state
  intAcknow_step=step_value;
  intAcknow_proxi=Prox_switch;
  strAcknowledge=strAcknow1+intAcknow_step+strAcknow2+intAcknow_proxi;
  Serial.println(strAcknowledge);    
// End of good measage and actions  
// Empty the inString
  inString="";
}

delay(5); 
// END 
}

